#include "../include/asset_mgr.hpp"
#include <PSX/File/Processor/cd_file_processor.hpp>
#include <PSX/AutoLBA/auto_lba.hpp>
#include <PSX/AutoLBA/auto_lba_declaration.hpp>
#include <stdio.h>

extern "C" uint32_t __gpu_tests_start;

namespace Assets {
    enum LBA {
        __jabyengine_start_lba_request
        __jabyengine_request_lba_for(PACO,         "ASSETS/MAIN/PACO.BIN"),
        __jabyengine_request_lba_for(GPU_TEST_OVL, "GTO.BIN"),
        __jabyengine_end_lba_request
    };
    __declare_lba_header(LBA);

    static void load(const CDFile* assets, size_t size) {
        const auto      buffer_cfg = CDFileProcessor::BufferConfiguration::new_default();
        CDFileProcessor file_processor;

        file_processor.setup(lba, CDFileProcessor::JobArray{assets, size}, buffer_cfg);
        while(true) {
            switch(file_processor.process()) {
                case Progress::InProgress:
                    break;

                case Progress::Done:
                    if(!file_processor.next(lba, buffer_cfg)) {
                        return;
                    }
                    break;
                
                case Progress::Error:
                    printf("Error detected! Aborting load\n");
                    return;
            }
        }
    }

    template<size_t N>
    static void load(const CDFile (&files)[N]) {
        return load(files, N);
    }

    namespace Main {
        static const CDFile Files[] = {
            CDFileBuilder::simple_tim(LBA::PACO, PacoTIM)
        };

        void load() {
            ::Assets::load(Files);
        }
    }

    namespace Overlay {
        static const CDFile Files[] = {
            CDFileBuilder::overlay(LBA::GPU_TEST_OVL, &__gpu_tests_start)
        };

        void load_gpu_test() {
            ::Assets::load(Files);
            // Load Overlay assets?!
        }
    }
}